@@ -143,91 +143,114 @@ class TreeInstanceManager(TreeClassManager):
     self._obj_ref      = weakref.ref(obj)
     self.class_manager = class_manager
 
+  @property
+  def pk(self):
+    return getattr(self._get_obj(), self._tree_options.pk_field.name)
+
+  @property
+  def parent_id(self):
+    return getattr(self._get_obj(), self._tree_options.parent_id_field.name)
+
+  @property
+  def parent(self):
+    return getattr(self._get_obj(), self._tree_options.parent_field_name)
+
+  @property
+  def tree_id(self):
+    return getattr(self._get_obj(), self._tree_options.tree_id_field.name)
+
+  @property
+  def left(self):
+    return getattr(self._get_obj(), self._tree_options.left_field.name)
+
+  @property
+  def right(self):
+    return getattr(self._get_obj(), self._tree_options.right_field.name)
+
+  @property
+  def depth(self):
+    return getattr(self._get_obj(), self._tree_options.depth_field.name)
+
+  def filter_root_node(self):
+    """Return a filter condition identifying the root node of the tree which
+    includes this node."""
+    return self.filter_root_node_of_node(self._get_obj())
+
+  def query_root_node(self):
+    """Return a query containing the root node of the tree which includes this
+    node."""
+    return self.query_root_node_of_node(self._get_obj())
+
+  @property
+  def root_node(self):
+    "Return the root node of the tree which includes this node."
+    return self.query_root_node_of_node(self._get_obj()).one()
+
+  def filter_ancestors(self, include_self=False):
+    "The same as :meth:`filter_descendants` but filters ancestor nodes."
+    return self.filter_ancestors_of_node(self._get_obj(), include_self=include_self)
+
+  def query_ancestors(self, session=None, include_self=False):
+    "The same as :meth:`query_descendants` but queries node's ancestors."
+    return self.query_ancestors_of_node(self._get_obj(), session=session, include_self=include_self)
+
   def filter_parent(self):
     "Get a filter condition for a node's parent."
-    options = self._tree_options
-    obj = self._get_obj()
-    #self._get_session_and_assert_flushed(obj)
-    parent_id = getattr(obj, self.parent_id_field.name)
-    if parent_id is None:
-        return sqlalchemy.sql.literal(False)
-    filter_ = self.pk_field == parent_id
-    return filter_
+    return self.filter_parent_of_node(self._get_obj())
 
-  def filter_ancestors(self, and_self=False):
-    "The same as :meth:`filter_descendants` but filters ancestor nodes."
-    options = self._tree_options
-    obj     = self._get_obj()
+  def filter_siblings(self, include_self=False):
+    "Get a filter condition for a node's siblings."
+    return self.filter_siblings_of_node(self._get_obj(), include_self=include_self)
 
-    #self._get_session_and_assert_flushed(obj)
+  def query_siblings(self, session=None, include_self=False):
+    "Get a query containing a nodes siblings."
+    return self.query_siblings_of_node(self._get_obj(), session=session, include_self=include_self)
 
-    # Restrict ourselves to just those nodes within the same tree:
-    tree_id = getattr(obj, self.tree_id_field.name)
-    filter_ = self.tree_id_field == tree_id
+  def filter_previous_siblings(self, include_self=False):
+    "Get a filter condition for the siblings of a node which occur prior to it in tree ordering."
+    return self.filter_previous_siblings_of_node(self._get_obj(), include_self=include_self)
 
-    alias = sqlalchemy.alias(options.table)
-    left_field = self.left_field
-    filter_ &= sqlalchemy.between(
-      getattr(alias.c, self.left_field.name),
-      self.left_field, self.right_field)
-    filter_ &= getattr(alias.c, self.pk_field.name) == \
-               getattr(obj,     self.pk_field.name)
+  def query_previous_siblings(self, session=None, include_self=False):
+    "Get a query containing the siblings of a node which occur prior to it in tree ordering."
+    return self.query_previous_siblings_of_node(self._get_obj(), session=session, include_self=include_self)
 
-    if not and_self:
-      filter_ &= self.pk_field != getattr(obj, self.pk_field.name)
+  def filter_next_siblings(self, include_self=False):
+    "Get a filter condition for the siblings of a node which occur subsequent to it in tree ordering."
+    return self.filter_next_siblings_of_node(self._get_obj(), include_self=include_self)
 
-    # WHERE tree_id = <node.tree_id> AND <node.path> LIKE path || '%'
-    #filter_ = (self.tree_id_field == tree_id) \
-    #          & sqlalchemy.sql.expression.literal(
-    #                path, sqlalchemy.String
-    #            ).like(options.path_field + '%')
-    #if and_self:
-    #  filter_ &= self.depth_field  <= depth
-    #else:
-    #  filter_ &= self.depth_field < depth
-    return filter_
+  def query_next_siblings(self, session=None, include_self=False):
+    "Get a query containing the siblings of a node which occur subsequent to it in tree ordering."
+    return self.query_next_siblings_of_node(self._get_obj(), session=session, include_self=include_self)
 
-  def query_ancestors(self, session=None, and_self=False):
-    "The same as :meth:`query_descendants` but queries node's ancestors."
-    query = self._get_query(self._get_obj(), session) \
-                .filter(self.filter_ancestors(and_self=and_self))
-    return query
+  @property
+  def previous_sibling(self):
+    "Returns the previous sibling with respect to tree ordering, or `None`."
+    if self.is_root_node:
+      ordering = sqlalchemy.sql.expression.desc(self.tree_id_field)
+    else:
+      ordering = sqlalchemy.sql.expression.desc(self.left_field)
+    return self.query_previous_siblings().order_by(ordering).first()
+
+  @property
+  def next_sibling(self):
+    "Returns the next sibling with respect to tree ordering, or `None`."
+    if self.is_root_node:
+      ordering = sqlalchemy.sql.expression.asc(self.tree_id_field)
+    else:
+      ordering = sqlalchemy.sql.expression.asc(self.left_field)
+    return self.query_next_siblings().order_by(ordering).first()
 
   def filter_children(self):
     """The same as :meth:`filter_descendants` but filters direct children only
-    and does not accept an :attr:`and_self` parameter."""
-    options = self._tree_options
-    obj     = self._get_obj()
-
-    # We don't actually need the session object, but we need to make sure the
-    # object is bound and that its tree fields are all filled out, which this
-    # method checks for:
-    #self._get_session_and_assert_flushed(obj)
-
-    # Since we store the denormalized depth field, this query is pretty easy.
-    # Just ask for those descendants with the correct depth value.
-    depth = getattr(obj, self.depth_field.name) + 1
-
-    # Oh yeah, using adjacency relation may be more efficient here. But one
-    # can access AL-based children collection without this library at all. And
-    # in this case we can be sure that at least the `(tree_id, left, right)`
-    # index is used. `parent_id` field may not have index set up so condition
-    # `pk == parent_id` in a SQL query could be even less efficient.
-    filter_ = self.filter_descendants() & (self.depth_field == depth)
-
-    # We're done!
-    return filter_
+    and does not accept an :attr:`include_self` parameter."""
+    return self.filter_children_of_node(self._get_obj())
 
   def query_children(self, session=None):
     """The same as :meth:`query_descendants` but queries direct children only
-    and does not accept an :attr:`and_self` parameter."""
-    # This simply builds on other methods to return a query with the
-    # appropriate filter clause already applied:
-    query = self._get_query(self._get_obj(), session) \
-                .filter(self.filter_children())
-    return query
+    and does not accept an :attr:`include_self` parameter."""
+    return self.query_children_of_node(self._get_obj(), session=session)
 
-  def filter_descendants(self, and_self=False):
+  def filter_descendants(self, include_self=False):
     """Get a filter condition for node's descendants.
 
     Requires that node has `tree_id`, `left`, `right` and `depth` values
@@ -244,40 +267,15 @@ class TreeInstanceManager(TreeClassManager):
     `filter_descendants`, don't use it for such purpose as there is a better
     way for such simple queries: :meth:`query_descendants`.
 
-    :param and_self:
+    :param include_self:
       `bool`, if set to `True`, include this node in the filter as well.
     :return:
       a filter clause applicable as argument for
       `sqlalchemy.orm.Query.filter()` and others.
     """
-    options = self._tree_options
-    obj     = self._get_obj()
+    return self.filter_descendants_of_node(self._get_obj(), include_self=include_self)
 
-    # We don't actually need the session object, but we need to make sure the
-    # object is bound and that its tree fields are all filled out, which this
-    # method checks for:
-    #self._get_session_and_assert_flushed(obj)
-
-    tree_id = getattr(obj, self.tree_id_field.name)
-    left    = getattr(obj, self.left_field.name)
-    right   = getattr(obj, self.right_field.name) - 1
-
-    # Restrict ourselves to just those nodes within the same tree:
-    filter_ = self.tree_id_field == tree_id
-
-    # If the caller requests the specified node to be included, this is most
-    # easily accomplished by decrementing left by one, so that the node is now
-    # included in the resulting interval:
-    if not and_self:
-      left = left + 1
-    # Any node which has a left value between this node's left and right
-    # values must be a descendant of this node:
-    filter_ &= sqlalchemy.between(self.left_field, left, right)
-
-    # Filter is complete!
-    return filter_
-
-  def query_descendants(self, session=None, and_self=False):
+  def query_descendants(self, session=None, include_self=False):
     """Get a query for node's descendants.
 
     Requires that node is in “persistent” state or in “pending” state in
@@ -288,17 +286,12 @@ class TreeInstanceManager(TreeClassManager):
       node is in “detached” state and :attr:`session` is not provided, query
       will be detached too (will require setting `session` attribute to
       execute).
-    :param and_self:
+    :param include_self:
       `bool`, if set to `True` self node will be selected by query.
     :return:
-      a `sqlalchemy.orm.Query` object which contains only node's descendants
-      and is ordered by `path`.
+      a `sqlalchemy.orm.Query` object which contains only node's descendants.
     """
-    # This simply builds on other methods to return a query with the
-    # appropriate filter clause already applied:
-    query = self._get_query(self._get_obj(), session) \
-                .filter(self.filter_descendants(and_self=and_self))
-    return query
+    return self.query_descendants_of_node(self._get_obj(), session=session, include_self=include_self)
 
   def get_descendant_count(self):
     "Returns the number of descendants this node has."
@@ -313,6 +306,77 @@ class TreeInstanceManager(TreeClassManager):
 
     return (right - left - 1) / 2
 
+  def filter_leaf_nodes(self, include_self=False):
+    """Creates a filter containing leaf nodes of this node instance.
+
+    Requires that node has `tree_id`, `left`, `right` and `depth` values
+    available (that means it has “persistent version” even if the node itself
+    is in “detached” state or it is in “pending” state in `autoflush`-enabled
+    session).
+
+    :param include_self:
+      `bool`, if set to `True`, the filter will also include this node (if it
+      is a leaf node).
+    """
+    return self.filter_leaf_nodes_of_node(self._get_obj(), include_self=include_self)
+
+  def query_leaf_nodes(self, session=None, include_self=False):
+    """Returns a query containing leaf nodes of this node instance.
+
+    Requires that node has `tree_id`, `left`, `right` and `depth` values
+    available (that means it has “persistent version” even if the node itself
+    is in “detached” state or it is in “pending” state in `autoflush`-enabled
+    session).
+
+    :param session:
+      session object for query. If not provided, node's session is used. If
+      node is in “detached” state and :attr:`session` is not provided, query
+      will be detached too (will require setting `session` attribute to
+      execute).
+    :param include_self:
+      `bool`, if set to `True`, the filter will also include this node (if it
+      is a leaf node).
+    :return:
+      a `sqlalchemy.orm.Query` object which contains only node's descendants
+      which are themselves leaf nodes.
+    """
+    return self.query_leaf_nodes_of_node(self._get_obj(), session=session, include_self=include_self)
+
+  @property
+  def is_root_node(self):
+    "Returns `True` if the node has no parent."
+    return self.left == 1
+
+  @property
+  def is_child_node(self):
+    "Returns `True` if the node has a parent."
+    return self.left != 1
+
+  @property
+  def is_leaf_node(self):
+    "Returns `True` if the node has no children."
+    return self.left == self.right-1
+
+  def is_ancestor_of(self, descendant, include_self=False):
+    "Returns `True` if the passed-in node is a descendant of this node."
+    return self.any_ancestors_of(
+      descendant, self._get_obj(), include_self=include_self)
+
+  def is_sibling_of(self, sibling, include_self=True):
+    "Returns `True` if the passed-in node is a sibling to this node."
+    if self.parent_id != getattr(sibling, self.parent_id_field.name):
+      return False
+    return (include_self and True
+                          or self.pk != getattr(sibling, self.pk_field.name))
+
+  def is_child_of(self, parent):
+    "Returns `True` if the passed-in node is parent to this node."
+    return self.parent_id == getattr(parent, self.pk_field.name)
+
+  def is_descendant_of(self, ancestor, include_self=False):
+    "Returns `True` if the passed-in node is an ancestor of this node."
+    return self.any_descendants_of(ancestor, self._get_obj(), include_self=include_self)
+
   def _get_obj(self):
     "Dereference weakref and return node instance."
     return self._obj_ref()
